@@ -1,6 +1,6 @@
 # calibcant - tools for thermally calibrating AFM cantilevers
 #
-# Copyright (C) 2008-2012 W. Trevor King <wking@drexel.edu>
+# Copyright (C) 2008-2013 W. Trevor King <wking@tremily.us>
 #
 # This file is part of calibcant.
 #
@@ -575,7 +575,7 @@ class Calibrator (object):
         return _load(filename=filename, group=group, specs=specs)
 
     def load_raw(self, filename=None, group='/'):
-        """Load results saved during `.aquire()` by bumps, etc."""
+        """Load results saved during `.acquire()` by bumps, etc."""
         data = {}
         with _h5py.File(filename, 'r') as f:
             for name,loader in [('bump',_bump_load),
